from alegra.resources.contact import Contact
from alegra.resources.invoice import Invoice
from alegra.resources.item import Item
from alegra.resources.retention import Retention
from alegra.resources.tax import Tax



from alegra.api_requestor import APIRequestor


class APIResource:
    @classmethod
    def retrieve(cls, resource_id, user=None, token=None, api_base=None,
                 api_version=None, **params):
        requestor = APIRequestor(
            user=user,
            token=token,
            api_base=api_base,
            api_version=api_version,
        )
        url = cls.class_url() + str(resource_id)
        response = requestor.request(
            method="get",
            url=url,
            params=params,
        )
        return response

    @classmethod
    def class_url(cls):
        if cls == APIResource:
            raise NotImplementedError(
                "APIResource is an abstract class. You should perform "
                "actions on its subclasses (e.g. Contact, Invoice)"
            )
        return "{}/".format(cls.OBJECT_NAME)



from alegra.resources.abstract import CreateableAPIResource
from alegra.resources.abstract import DeleteableAPIResource
from alegra.resources.abstract import ListableAPIResource
from alegra.resources.abstract import UpdateableAPIResource


class Contact(
    CreateableAPIResource,
    DeleteableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "contacts"



from alegra.resources import Contact
from alegra.resources import Invoice
from alegra.resources import Item
from alegra.resources import Retention
from alegra.resources import Tax


user = None
token = None
api_base = "https://api.alegra.com/api"
api_version = "v1"



from alegra.api_requestor import APIRequestor
from alegra.resources.abstract import CreateableAPIResource 
from alegra.resources.abstract import EmailableAPIResource
from alegra.resources.abstract import ListableAPIResource
from alegra.resources.abstract import UpdateableAPIResource
from alegra.resources.abstract import VoidableAPIResource


class Invoice(
    CreateableAPIResource,
    EmailableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    VoidableAPIResource,
):
    OBJECT_NAME = "invoices"

    @classmethod
    def open(cls, resource_id, user=None, token=None, api_base=None,
             api_version=None, **json):
        requestor = APIRequestor(
            user=user,
            token=token,
            api_base=api_base,
            api_version=api_version,
        )
        url = cls.class_url() + str(resource_id) + "/open/"
        response = requestor.request(
            method="post",
            url=url,
            json=json,
        )
        return response



from alegra.resources.abstract.api_resource import APIResource
from alegra.resources.abstract.createable_api_resource import CreateableAPIResource
from alegra.resources.abstract.deletable_api_resource import DeleteableAPIResource
from alegra.resources.abstract.emailable_api_resource import EmailableAPIResource
from alegra.resources.abstract.listable_api_resource import ListableAPIResource
from alegra.resources.abstract.updateable_api_resource import UpdateableAPIResource
from alegra.resources.abstract.voidable_api_resource import VoidableAPIResource



import os
import setuptools


here = os.path.abspath(os.path.dirname(__file__))
os.chdir(here)

with open(
    os.path.join(here, "LONG_DESCRIPTION.rst"), "r", encoding="utf-8"
) as fp:
    long_description = fp.read()

setuptools.setup(
    author="Chaty",
    author_email="developer@okchaty.com",
    name="alegra",
    license="MIT",
    description="alegra is a python package for connect to the Alegra's API.",
    version="0.1.1",
    long_description=long_description,
    url="https://github.com/okchaty/alegra",
    packages=setuptools.find_packages(exclude=["tests", "tests.*"]),
    python_requires=">=3.5",
    install_requires=["requests"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
	"Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
	"Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Intended Audience :: Developers",
    ],
)



import alegra
import base64

from requests import request


class APIRequestor(object):
    def __init__(self, user=None, token=None, api_base=None, api_version=None):
        self.user = user or alegra.user
        self.token = token or alegra.token
        self.api_base = api_base or alegra.api_base
        self.api_version = api_version or alegra.api_version
        self.api_url = "{}/{}".format(self.api_base, self.api_version)

    def authorization_header(self):
        """Returns authorization header."""
        user_token = "{}:{}".format(self.user, self.token).encode()
        authorization_code = base64.b64encode(user_token).decode("utf-8")
        return "Basic {}".format(authorization_code)

    def request(self, method, url, **kwargs):
        """Injects auth headers."""
        headers = {
            "Authorization": self.authorization_header(),
            "content-type": "application/json",
        }
        headers.update(kwargs.pop("headers", {}))  # Updates headers.
        # More info:
        # https://requests.readthedocs.io/en/master/api/#requests.request
        return request(
            method,
            url="{}/{}".format(self.api_url, url),
            headers=headers,
            **kwargs
        )



from alegra.resources.abstract.api_resource import APIResource
from alegra.api_requestor import APIRequestor


class ListableAPIResource(APIResource):
    @classmethod
    def list(cls, user=None, token=None, api_base=None, api_version=None,
             **params):
        requestor = APIRequestor(
            user=user,
            token=token,
            api_base=api_base,
            api_version=api_version,
        )
        url = cls.class_url()
        response = requestor.request(
            method="get",
            url=url,
            params=params,
        )
        return response
